/**
 * @ingroup   browse68_devel
 * @file      SC68html.c
 * @author    Ben(jamin) Gerard <ben@sashipa.com>
 * @date      1999/02/27
 * @brief     sc68 Browse Info HTML generator
 * @version   $Id: SC68html.c,v 1.1.1.1 2001/12/24 20:50:10 ben Exp $
 */

/*
 *                      sc68 - Browse Info HTML generator
 *         Copyright (C) 2001 Ben(jamin) Gerard <ben@sashipa.com>
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <ctype.h>

#include "browse68/SC68html.h"
#include "file68/SC68error.h"
#include "file68/SC68utils.h"
#include "file68/SC68os_general.h"

typedef unsigned int html_color_t;

typedef enum
{
  html_italic,
  html_bold,
  html_underline,
  html_strike,
  html_strong,
  html_em,
  html_address,
  html_code,
  html_cite,
  html_tt,
  html_var,
  html_samp,
  html_listing,
} html_style_e;


static char *bkg_pic = "pic/SC68tile.gif";
static int bkg_color = 0x303030;
static int text_color = 0x303030;
static int link_color = 0x40FFFF;
static int alink_color = 0xFF40FF;
static int basefont_size = 4;
static int h1_color = 0x8080C0;
static int h2_color = 0xD08080;
static int h3_color = 0x80C080;
static char *author_name =	"<font color=\"#D08080\">"
														"<a href=\"about.html#author\">BeN</a>"
														" / "
														"<a href=\"http//www.multimania.com/sashipa\">SasHipA</a>"
														"</font>";

/* Header format string parms :
 * - Intro comment
 * - Page title
 * - Author
 */
static void html_head(FILE *f, char *head_comment, char *title, char *author)
{
  fprintf(f,
	  "<html> <! %s >\n\n"
	  "<head>\n"
	  " <title>%s</title>\n"
	  " <meta name=\"author\" content=\"%s\">\n"
	  "</head>\n\n",
	  head_comment,title,author);
}

static void html_end(FILE *f)
{
  fprintf(f,"</body></html>\n");
}

/*
 */
static void html_style(FILE *f,
											char *bkg_pic, int bkg_color,
											int text_color,
											int link_color, int alink_color,
											int basefont_size)
{
	fprintf(f,
		"<body\n"
		"background=\"%s\"\n"
		"bgcolor=\"#%06x\"\n"
		"text=\"#%06x\"\n"
		"link=\"#%06x\"\n"
		"alink=\"#%0x\">\n"
		"<basefont size=%d>\n\n",
		 bkg_pic, bkg_color,
		 text_color,
		 link_color, alink_color,
		 basefont_size);
}

static void html_h1(FILE *f, char *format, ...)
{
  va_list list;
	va_start(list,format);
	fprintf(f,
	  "<h1><center><font color=\"#%06x\">\n"
	  "<u>",h1_color);
	vfprintf(f,format,list);
	fprintf(f,
	  "</u>\n"
	  "</font></center></h1>\n");
	va_end(list);
}

static void html_h2(FILE *f, char *format, ...)
{
  va_list list;
	va_start(list,format);
	fprintf(f,
	  "<hr><h2><center><font color=\"#%06x\">\n"
	  "<u>",h2_color);
	vfprintf(f,format,list);
	fprintf(f,
	  "</u>\n"
	  "</font></center></h2>\n");
	va_end(list);
}

static void html_h3(FILE *f, char *format, ...)
{
  va_list list;
	va_start(list,format);
	fprintf(f,
	  "<hr><h3><center><font color=\"#%06x\">\n"
	  ,h3_color);
	vfprintf(f,format,list);
	fprintf(f,
	  "</font></center></h3>\n");
	va_end(list);
}

static int generate_hd(FILE *f, SC68browse_info_t *bi)
{
	html_head(f,
	  "SC68 browse info - HTML document generated by sc68",
	  "SC68 Browser",
	  "BeN(jamin) Gerard / SasHipA");

	html_style(	f,
		bkg_pic,bkg_color,
		text_color,
		link_color,alink_color,
		basefont_size);

	html_h1(f,
		"SC68 - Browse Info<br>\n</u><h2>by %s</h2>",
		author_name);

  return 0;
}

/* Generate a string with char convertion to HTML
 * !! First call init conversion table !!
 */
static int generate_string(FILE *f, char *s)
{
  fprintf(f,s);
  return 0;
}

static int localtype = 0;

/* generate
 *  <a href="local://ref"> text </a>
 */
static int generate_localhref(FILE *f, char *ref, char *text)
{
  if(!localtype)
    fprintf(f,"<a href=\"local://");
  else
    fprintf(f,"<a href=\"file://");
  SC68os_fput_fname(f,ref);
   fprintf(f,"\">");
  generate_string(f,text);
  fprintf(f,"</a>\n");
  return 0;
}

static int generate_entry(FILE *f, SC68browse_file_t *be)
{
  fprintf(f," <LI>");
  if(be==NULL) return -1;
  generate_localhref(f,be->path,be->mname);
  return 0;
}

/* Returns # different author
 */
static int count_author(SC68browse_file_t *be)
{
  char *a;
  int  i;
  if(be==NULL) return 0;
  a = be->aname;
  for(i=0; be!=NULL && !strcmp(a,be->aname); be=be->nxt, i++);
  return i;
}

/* Make sub-Browse info ...
 */
static SC68browse_info_t *sub_bi(SC68browse_info_t *bi, SC68browse_file_t *be)
{
  char *a;
  memset(bi,0,sizeof(*bi));
  if(be==NULL) return bi;
  bi->head = be;
  bi->nauthor = 1;
  a = be->aname;
  for(; be!=NULL && !strcmp(a,be->aname); be=be->nxt)
  {
  	bi->n++;
    bi->ntrack  += be->ntrack;
    bi->natari  += !!(be->flags&(SC68BROWSE_YM|SC68BROWSE_MW));
    bi->namiga  += !!(be->flags&(SC68BROWSE_PL));
    bi->time    += be->time;
    bi->time    += be->time;
  };
  return bi;
}



/*
 * Generate an author string :
 *  author_name ( # files )
 */
static SC68browse_file_t *generate_author_entries( FILE *f,
                                           SC68browse_file_t *be)
{
  SC68browse_info_t bi;
  char *author;

  if(be==NULL)
    return NULL;

  if(sub_bi(&bi, be)==NULL)
  	return NULL;

  fprintf(f,"<a name=\"%s\"></a>\n",be->aname);
  html_h3(f,
  	"%s<br>"
  	"%d file%s (%d Atari ST, %d Amiga)<br>"
  	"%d track%s<br>"
  	"%s of music time",
  	be->aname,
  	bi.n, bi.n>1?"s":"",bi.natari,bi.namiga,
  	bi.ntrack, bi.ntrack>1?"s":"",
  	SC68utils_make_big_playing_time(bi.time)
  	);
  for(author=be->aname; be!=NULL && !strcmp(author,be->aname); be=be->nxt)
    generate_entry(f,be);
/*  fprintf(f,"</ul>\n\n");*/
  fflush(f);
  return be;
}


static int generate_body(FILE *f, SC68browse_info_t *bi)
{
  int cnt;
  SC68browse_file_t *be;

  if(f==NULL || bi==NULL)
    return -1;

  for(be=bi->head,cnt=0; be!=NULL; be=generate_author_entries(f,be));
  fflush(f);
  return 0;
}


/* Short Cut : List of sorted author Version
 */
static int generate_author_shortcut_list(FILE *f,SC68browse_info_t *bi)
{
  SC68browse_file_t *be;
  int               i;
  char              *old_author = "";
  fprintf(f,"<center>\n");
  for(i=0, be=bi->head; be!=NULL; be=be->nxt)
  {
    if( strcmp(be->aname,old_author) )
    {
      old_author = be->aname;
      fprintf(f," - <a href=\"#%s\">%s</a>\n",be->aname,be->aname);
    }
  }
  fprintf(f," -\n</center>\n\n");
  return 0;
}

/* Short Cut : Table of sorted author Version
 */
static int generate_author_shortcut_table(FILE *f,SC68browse_info_t *bi)
{
  SC68browse_file_t *be;
  int               i;
  char              *old_author = "";
  int col=0,maxcol=5;

	fprintf(f,
		"<center>\n"
		"<table border>\n"
		"<caption align=\"top\">Authors Table</caption>\n");
  for(i=0, be=bi->head; be!=NULL; be=be->nxt)
  {
    if( strcmp(be->aname,old_author) )
    {
      old_author = be->aname;
      fprintf(f,"%s<td><a href=\"#%s\">%s</a></td>%s\n",
      	!col?"<tr>":"    ",be->aname,be->aname,col+1==maxcol?"</tr>":"");
      col = (col+1) % maxcol;
    }
  }
  fprintf(f,"%s</table></center>\n\n",col?"</tr>":"");
  return 0;
}


/* Generate Short cut Hreference to wrap
 * directly to an author
 */
static int generate_shortcut(FILE *f,SC68browse_info_t *bi)
{
/*	html_h3(f,"%d author%s<br>%d file%s (%d Atari ST, %d Amiga)<br>%s of music time",
          bi->nauthor, bi->nauthor>1?"s":"",
          bi->n, bi->n>1?"s":"", bi->natari, bi->namiga,
          SC68utils_make_big_playing_time(bi->time));*/
	html_h3(f,
  	"%d author%s<br>"
  	"%d file%s (%d Atari ST, %d Amiga)<br>"
  	"%d track%s<br>"
  	"%s of music time",
  	bi->nauthor, bi->nauthor>1?"s":"",
  	bi->n, bi->n>1?"s":"",bi->natari,bi->namiga,
  	bi->ntrack, bi->ntrack>1?"s":"",
  	SC68utils_make_big_playing_time(bi->time)
  	);


/*	generate_author_shortcut_list(f,bi);*/
	generate_author_shortcut_table(f,bi);
  fflush(f);
  return 0;
}

/*  Generate a HTML page in a "wt" opened FILE with
 *  a Browse Info DataBase
 */
int SC68html_generate(FILE *f, SC68browse_info_t *bi)
{
  if(f==NULL || bi==NULL)
    return SC68error_add("SC68html_generate(%p,%p) NULL pointer",f,bi);

  generate_hd(f,bi);
  generate_shortcut(f,bi);
  generate_body(f,bi);
  html_end(f);
  fflush(f);

  return 0;
}

/*  SC68html_setlocal(int type) :
 *   type == 0 local is HREF="local: ( DEFAULT )
 *   type != 0 local is HREF="file:
 */
void SC68html_setlocal( int type )
{
  localtype = !!type;
}

